/*
 * Copyright 2022 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package androidx.compose.material3.windowsizeclass

import android.app.Activity
import androidx.compose.runtime.Composable
import androidx.compose.ui.graphics.toComposeRect
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.platform.LocalDensity
import androidx.window.layout.WindowMetricsCalculator

/**
 * Calculates the window's [SizeClass] for this [Activity].
 *
 * A new [SizeClass] will be returned whenever a configuration change causes the width or height of
 * the window to cross a breakpoint, such as when the device is rotated or the window is resized.
 *
 * @sample androidx.compose.material3.windowsizeclass.samples.AndroidSizeClassSample
 */
@ExperimentalMaterial3WindowSizeClassApi
@Composable
fun Activity.calculateSizeClass(): SizeClass {
    // Observe view configuration changes and recalculate the size class on each change. We can't
    // use Activity#onConfigurationChanged as this will sometimes fail to be called on different
    // API levels, hence why this function needs to be @Composable so we can observe the
    // ComposeView's configuration changes.
    LocalConfiguration.current
    val density = LocalDensity.current
    val metrics = WindowMetricsCalculator.getOrCreate().computeCurrentWindowMetrics(this)
    val size = with(density) { metrics.bounds.toComposeRect().size.toDpSize() }
    return SizeClass.calculateFromSize(size)
}
